from urllib.parse import urlparse, urlunsplit
import requests
import socket
import sys
import traceback
import time
from forcediphttpsadapter.adapters import ForcedIPHTTPSAdapter


__version__ = "0.0.2"
PREFERRED_TYPE = "text/html"
TIMEOUT = 2


class BotError(Exception):
    pass


class WebBot:
    def getUserAgent(self):
        return "%s/%s (+%s)" % (
            "URLCHECKER",
            __version__,
            "https://lab.nexedi.com/romain/url-checker",
        )

    def request(
        self,
        method,
        url,
        headers=None,
        stream=False,
        timeout=TIMEOUT,
        allow_redirects=False,
        verify=True,
        session=requests,
        **kwargs,
    ):

        if headers is None:
            headers = {}
        if "Accept" not in headers:
            headers["Accept"] = "%s;q=0.9,*/*;q=0.8" % PREFERRED_TYPE
        if "User-Agent" not in headers:
            # XXX user agent
            headers["User-Agent"] = self.getUserAgent()

        kwargs["stream"] = stream
        kwargs["timeout"] = timeout
        kwargs["allow_redirects"] = allow_redirects
        kwargs["verify"] = verify
        args = [method, url]

        kwargs["headers"] = headers

        try:
            response = session.request(*args, **kwargs)
        except requests.exceptions.SSLError:
            # XXX Enter into unknown host
            response = requests.models.Response()
            response.status_code = 526
        except requests.exceptions.ConnectionError:
            response = requests.models.Response()
            response.status_code = 523
        except requests.exceptions.Timeout:
            response = requests.models.Response()
            response.status_code = 524
        except requests.exceptions.TooManyRedirects:
            response = requests.models.Response()
            response.status_code = 520
        except:
            # XXX Put it in body
            print(traceback.print_exception(*(sys.exc_info())))
            response = requests.models.Response()
            response.status_code = 0

        return response

    def check(self, url):
        parsed_url = urlparse(url)
        response = self.request("GET", url)
        print(url, response.status_code)

        # Get the list of available IPv4 frontend CDN
        hostname = parsed_url.hostname
        dns_info_list = socket.getaddrinfo(hostname, "http", socket.AF_INET)
        ip_list = [x[4][0] for x in dns_info_list]

        for ip in ip_list:
            session = requests.Session()
            # SNI Support
            if parsed_url.scheme == "https":
                # Provide SNI support
                base_url = urlunsplit(
                    (parsed_url.scheme, parsed_url.netloc, "", "", "")
                )
                session.mount(base_url, ForcedIPHTTPSAdapter(dest_ip=ip))

            response = self.request(
                "GET", url, headers={"Host": hostname}, session=session
            )

            print(ip, hostname, response.status_code)

    def run(self, url):
        print(time.strftime("%Y-%m-%d %H:%M:%S"))
        try:
            self.check(url)
        except KeyboardInterrupt:
            self.stop()
        except:
            print("Oups, error")
            raise
